require "refile/nobrainer"

module Nukleus
    class Document
        include NoBrainer::Document
        include NoBrainer::Document::Timestamps
        extend Refile::NoBrainer::Attachment

        after_destroy :remove_all_versions

        # A document can exist in many versions. Each has its own set of meta data and its own document.
        field       :document_id,                 :type => String, :required => true, :index => true
        field       :document_version,            :type => Integer, :required => true, :default => 0, :validates => {:uniqueness => {:scope => :document_id}}

        # The attached file
        attachment  :document_file
        field       :document_file_filename,      :type => String
        field       :document_file_size,          :type => Integer
        field       :document_file_content_type,  :type => String

        # Meta-Data
        field       :tags,                        :type => Array
        field       :meta_data,                   :type => Hash

        # Update with creating a copy with a higher version number.
        # @return returns the newly created and modified copy of the document or the current document on error.
        def update_with_copy(params)
            success = true
            if params.include?(:document_id)
                # Remove document_id from params in order to avoid unexpected duplications.
                # This should be an error. But if we expect not to have this param, we will get a problem
                # in DocumentController::document_params  due to an empty Document-Hash.
                params.delete(:document_id)
            end
            if params.include?(:document_version)
                errors.add(:document_version, "document_version cannot be changed")
                success = false
            end

            if success
                # make a deep copy of the current document and update data with this new version
                new_document_hash = self.raw_attributes
                # Remove old Document data. Otherwise we will see strange effects.
                new_document_hash.except!( :id, :document_file_id, :document_file_filename, :document_file_size, :document_file_content_type )
                new_document_hash[:document_version] += 1
                new_document = Document.new(new_document_hash)
                success = new_document.update(params)
            end

            # The original implementation returns true or false!
            success ? new_document : self
        end

        # This overrides the Nukleus::Document.where()
        # Only provide the latest version of each document (default). Use :all_document_versions => true to return all.
        def self.where(given_params)
            all_versions = given_params[:all_document_versions] || false
            params = given_params.except!(:all_document_versions)

            documents = super(params).order_by(:document_id, :document_version)

            unless all_versions
                max_doc_hash = {}
                documents.each do |document|
                    max_document = max_doc_hash[document.document_id]
                    if max_document.nil?
                        # Element not in hash: Just add it
                        max_doc_hash[document.document_id] = document
                    else
                        # Replace document in max_doc_hash if it is the latest document
                        if document.document_version > max_document.document_version
                            max_doc_hash[document.document_id] = document
                        end
                    end
                end

                max_doc_hash.values
            else
                documents
            end
        end

        private
        def remove_all_versions
            # Remove all documents with this document_id
            Document.where(document_id: self.document_id).each { |d| d.destroy }
            true
        end
    end
end
